# -*- coding: utf-8 -*-
"""
Created on Mon Sep 10 13:04:50 2018

@author: lolo
"""

import numpy as np
import matplotlib.pyplot as plt


# Para buscar archivos en tu carpeta:
from glob import glob

# Ejemplos de leer y escribir archivos de datos:
#    https://marceluda.github.io/python-para-fisicos/tuto/repositorio/IO/


#%% Función auxiliar


# Esta función es para corregir cuando se escribe con coma española en lugar de inglesa
def fix_comma(s):
    "Toma un texto y devuelve un float, corrigiendo la coma española como punto inglés"
    s = s.decode().replace(',','.')
    return float(s)

conversores = {0:fix_comma,1:fix_comma}


#%% Lectura de UN archivo de datos del motion DAQ


# Nombre del archivo
fn = 'archivo1.dat'


# Leo el archivo fn
datos = np.genfromtxt(fn,skip_header=4,converters=conversores)

# Me quedo con las columnas como arrays
tiempo, volts = datos.T 

# Criterio de selección: que la derivada numérica sea mayor a 2 (positivo)
I_flancos_subida      = np.diff(volts) > 2

# Filtro usando los índices de ese criterio
tiempo_flancos_subida = tiempo[1:][I_flancos_subida]
volts_flancos_subida  = volts[ 1:][I_flancos_subida]


fig, ax  = plt.subplots(3,1, figsize=(10,8),  constrained_layout=True , sharex=True)

ax[0].plot(tiempo, volts)
ax[0].plot(tiempo[1:], np.diff(volts))

ax[1].plot(tiempo, volts)
ax[1].plot(tiempo_flancos_subida, volts_flancos_subida, 'o')

ax[2].plot(tiempo, volts)
ax[2].plot(tiempo_flancos_subida[::2], volts_flancos_subida[::2], 'o')

fig.suptitle(fn)
        


#%% Lectura de archivo de datos del motion DAQ


# Creo listas donde acumular los datos que genero
valores = []
errores = []


# Variable auxiliar para saber si grafico cada archivo o no
GRAFICAR = True


# Recorro todos los archivos, los leo, los proceso y extraigo datos
for fn in sorted(glob("*.dat")):
    
    # Leo el archivo fn
    datos = np.genfromtxt(fn,skip_header=4,converters=conversores)
    
    # Me quedo con las columnas como arrays
    tiempo, volts = datos.T 
    
    # Criterio de selección: que la derivada numérica sea mayor a 2 (positivo)
    I_flancos_subida      = np.diff(volts) > 2
    
    # Filtro usando los índices de ese criterio
    tiempo_flancos_subida = tiempo[1:][I_flancos_subida]
    volts_flancos_subida  = volts[ 1:][I_flancos_subida]
    
    
    if GRAFICAR:
        fig, ax  = plt.subplots(3,1, figsize=(10,8),  constrained_layout=True , sharex=True)
    
        ax[0].plot(tiempo, volts)
        ax[0].plot(tiempo[1:], np.diff(volts))
        
        ax[1].plot(tiempo, volts)
        ax[1].plot(tiempo_flancos_subida, volts_flancos_subida, 'o')
        
        ax[2].plot(tiempo, volts)
        ax[2].plot(tiempo_flancos_subida[::2], volts_flancos_subida[::2], 'o')
        
        fig.suptitle(fn)
        
    
    # Los periodos son la resta entre consecutivos de flancos de subida... pero salteandome de a dos
    vector_de_periodos = np.diff(tiempo_flancos_subida[::2])
    
    # Estadísticos
    valor_medio = vector_de_periodos.mean()
    valor_std   = vector_de_periodos.std()
    
    
    print(f'\n---------------------\n{fn}\n')
    print(f'Valor: {valor_medio}  +-  {valor_std}')
    
    valores.append( valor_medio )
    errores.append( valor_std )
    
    # OJO! Acá puse como error el STD ... pero el error de la media es STD/RAIZ(N)


#%% Grafico los datos acumulados

fig, ax  = plt.subplots(1,1, figsize=(10,8),  constrained_layout=True , sharex=True)

ax.errorbar( [0,1,2,3], valores , yerr=errores, 
            fmt='.', mfc='none', elinewidth = 2, capsize=4 , color='black',ms=15)
    
ax.set_xticks([0,1,2,3])
ax.set_xticklabels(sorted(glob("*.dat")))

ax.grid(True,ls=':', color='lightgray')
